#! /bin/awk -f
# ######################################################################### #
#                                                                           #
# File:     Filtre.awk                                                      #
# Authors:  Brigitte Bigi                                                   #
# Date:     november, 2003                                                  #
# Licence:  GPL                                                             #
# Brief:     Programme de filtrage des corpus du journal "Le Monde" de 2000 #
#             2002 (corpus fournit par ELRA).                              #
#            On marque les dbuts et fins d'articles par <ART> et </ART>.   #
#            Les thmes sont nots dans une ligne <T:theme1:theme2:...>.    #
#            Les dbuts et fins de paragraphes sont nots par <p> et </p>.  #
#            Les dbuts et fins de phrases <s> et </s> sont ajouts en      #
#            dbut et fin de paragraphes.                                   #
#                                                                           #
# ######################################################################### #


BEGIN{
	body = 0
}


# -------------------------
# Debut d'un nouvel article
# -------------------------
/^#ACC/{
	if ( body == 1)
		print "\n</s>\n</p>\n</ART>"
	print "<ART>"
	body = 0 
}

# --------------------------------
# Indication du titre d'un article
# --------------------------------
/^#TIJ/{
	gsub(/</, " ")
	gsub(/>/, " ")
	sub(/#TIJ\(1\)=/, "")
	print "<TITLE>"
	print
	print "</TITLE>"
	body = 0 
}

# ----------------------------------
# Indication des thmes d'un article
# ----------------------------------
/^#CAT/{
	sub(/#CAT\(1\)=/, "")
	gsub(/,/, "\n")
	print "<TOPICS>"
	print
	print "</TOPICS>"
	body = 0 

}

# -----------------------------------------
# Indication du dbut du corps d'un article
# -----------------------------------------
/^#TEX/ {
	sub(/#TEX=/, "")
	print "<p>\n<s>\n"
	body = 1
}

# ----------
# Autres cas
# ----------
{
	gsub (/<?#FIELD/, "")
	gsub (/<?#LINE/, "")
	gsub (/@ ARTICLE/, "")
	gsub (/@ ARTICL/, "")

 	# ------------------
	# Corps d'un article
	# ------------------
	if (body == 1)
	{
		# Marque de fin d'article
		gsub(/<[^>]/, "")

		# Marque de fin de ligne
		if (match($0,/[^<]>/) != 0 || match($0,/<>/) != 0)
		{
			gsub(/[^<]>/, "")
			gsub(/<>/, "\n</s>\n</p>\n<p>\n<s>\n")
			print		
		}
		else
		{
			gsub(/</, " ")
			gsub(/>/, " ")
			printf "%s", $0
		}
	}
}



