#! /bin/awk -f
# ######################################################################### #
#                                                                           #
# File:     RemoveWords.awk                                                 #
# Authors:  Brigitte Bigi                                                   #
# Date:     november, 2003                                                  #
# Licence:  GPL                                                             #
# Brief:    Load a vocabulary and remove these words of a corpus            #
#                                                                           #
# Parameters:                                                               #
#	fichier_vocab : file name of the vocabulary                         #
#                                                                           #
# ######################################################################### #
 



# ######################################################################### #
#                                                                           #
# fonction LoadVocab                                                        #
# 	Charge en mmoire un vocabulaire (fichier contenant un mot par      #
#       ligne tri dans l'ordre alpha-numerique ("sort" format POSIX)       #
#                                                                           #
# Paramtres:                                                               #
#       fichier_vocab est le nom du fichier du vocabulaire                  #
#                                                                           #
# Retour:                                                                   #
#       tableau tabvoc qui contient le vocabulaire                          #
#       renvoie la taille du tableau tabvoc (nb de mots du vocab)           #
#                                                                           #
# ######################################################################### #
function LoadVocab (fichier_vocab)
{
	ind = 0
	while ( getline < fichier_vocab )
	{
		# chaque ligne est stocke dans "tabvoc"
		tabvoc[ind]=$0
		ind++
	}
	return ind
}
# ######################################################################### #





# ######################################################################### #
#                                                                           #
# fonction IsOOV                                                            #
#	Recherche dichotoique d'un mot dans un tableau de mot ordonn       #
#                                                                           #
# Paramtres:                                                               #
#	tabvoc est un tableau de chaines de caractres                      #
#       mot est une chaine de caractre                                     #
#       ind est un entier qui reprsente la taille du tableau               #
#                                                                           #
# Retour:                                                                   #
#       0 si le mot est prsent, 1 s'il est absent                          #
#                                                                           #
# ######################################################################### #
function IsOOV (tabvoc,mot,ind)
{
	gauche = 0
	droit  = ind+1
	
	while ( gauche != droit )
	{
		median = int((droit+gauche)/2) 
		if ( tabvoc[median] > mot )
			droit = median
		else
		{
			if ( tabvoc[median] < mot )
				gauche = median+1
			else
			{
				return 0	
			}
		}
	}
	return 1
}
# ######################################################################### #
 



# ######################################################################### #
# Programme Principal                                                       #
# ######################################################################### #

BEGIN{
	ind = LoadVocab(fichier_vocab)
}

# ######################################################################### #

{

	for (i=1; i<=NF; i++)
	{
		if (IsOOV(tabvoc,$i,ind) == 1)
			print $i
	}
}

# ######################################################################### #



