#! /bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Tokenize.awk                                                    #
# Authors:  Brigitte Bigi                                                   #
# Date:     november, 2003                                                  #
# Licence:  GPL                                                             #
# Brief:    Tokenize a corpus by using a vocabulary and some rules          #
#                                                                           #
# Parameters:                                                               #
#       fichier_vocab : file name of the vocabulary                         #
#                                                                           #
# ######################################################################### #
  
 
 
 
# ######################################################################### #
#                                                                           #
# fonction LoadVocab                                                        #
#       Charge en mmoire un vocabulaire (fichier contenant un mot par      #
#       ligne tri dans l'ordre alpha-numerique ("sort" format POSIX)       #
#                                                                           #
# Paramtres:                                                               #
#       fichier_vocab est le nom du fichier du vocabulaire                  #
#                                                                           #
# Retour:                                                                   #
#       tableau tabvoc qui contient le vocabulaire                          #
#       renvoie la taille du tableau tabvoc (nb de mots du vocab)           #
#                                                                           #
# ######################################################################### #
function LoadVocab (fichier_vocab)
{
        ind = 0
        while ( getline < fichier_vocab )
        {
                # chaque ligne est stocke dans "tabvoc"
                tabvoc[ind]=$0
                ind++
        }
        return ind
}
# ######################################################################### #
 
 

# ######################################################################### #
#                                                                           #
# fonction IsOOV                                                            #
#       Recherche dichotomique d'un mot dans un tableau de mot ordonn      #
#                                                                           #
# Paramtres:                                                               #
#       tabvoc est un tableau de chaines de caractres                      #
#       mot est une chaine de caractre                                     #
#       ind est un entier qui reprsente la taille du tableau               #
#                                                                           #
# Retour:                                                                   #
#       0 si le mot est prsent, 1 s'il est absent                          #
#                                                                           #
# ######################################################################### #
function IsOOV (tabvoc,mot,ind)
{
        gauche = 0
        droit  = ind+1
                                                                                                                                           
        while ( gauche != droit )
        {
                median = int((droit+gauche)/2)
                if ( tabvoc[median] > mot )
                        droit = median
                else
                {
                        if ( tabvoc[median] < mot )
                                gauche = median+1
                        else
                        {
                                return 0
                        }
                }
        }
        return 1
}
# ######################################################################### #
                                                                                                                                           


# ######################################################################### #
#                                                                           #
# fonction Regroupe                                                         #
#	Ecrit les regroupements de squences de mots                        #
#                                                                           #
#                                                                           #
# Variables modifies :                                                     #
#	Lst_Mots est un tableau des mots  regrouper                        #
#       nb_mots est la taille de ce tableau                                 #
#                                                                           #
# ######################################################################### #
function Regroupe()
{
	# Incrment de boucle pour parcourir la liste de mots
	i = nb_mots

	# Index de fin du "token" trouv dans Lst_Mots 
	i_ok = 1

	while ( i >= 1 && i_ok == 1)
	{
		# On essai de regrouper tous les tokens
		token = Lst_Mots[1]
		if (i > 1)
		{
			for (k = 2; k <= i; k++)
				token = token "_" Lst_Mots[k]
			# Les tokens runis sont dans le lexique
			if ( IsOOV(tabvoc,token,ind) == 0 ) 
				i_ok = i
		}	
		i--
	}

	print token

	# shift Lst_Mots
	for (k=i_ok+1;k<=nb_mots;k++)
		Lst_Mots[k-i_ok] = Lst_Mots[k]
	nb_mots = nb_mots-i_ok
}
# ######################################################################### #



# ######################################################################### #
#                                                                           #
# Programme Principal                                                       #
#                                                                           #
# on va supposer que le regroupement le plus long contient 5 mots, c'est    #
# le cas de "au fur et  mesure"... qui devient "au_fur_et__mesure".       #
# On va stocker ces 5 mots dans une liste.                                  #
#                                                                           # 
# ######################################################################### #

BEGIN{
	ind = LoadVocab(fichier_vocab)	# Charge le vocab
	nb_mots = 0
}


{
	if ( match($0, /<[\/A-Zsp:]+>/))
	{
		while ( nb_mots > 0 )
			Regroupe()
		# Les lignes qui sont des balises doivent rester telles-quelles
		print
	}
	else
	{
			# On met le mot que l'on a lu dans la liste
			nb_mots = nb_mots + 1
			Lst_Mots[nb_mots] = $0

			# Quand la liste est pleine, on tente de regrouper le mot le
			# plus ancien avec ses suivants : 
			# 	- soit on regroupe, et on crit ce qui est regroup,
			#	- soit on crit le mot le plus ancien s'il ne peut
			#	  pas se regrouper
			if ( nb_mots == 5 )
				Regroupe()
	}
}

