#! /bin/awk -f
# ######################################################################### #
#                                                                           #
# File:     Tokenize.awk                                                    #
# Authors:  Brigitte Bigi                                                   #
# Date:     november, 2003                                                  #
# Licence:  GPL                                                             #
# Brief:    Tokenize a corpus by using a vocabulary and some rules          #
#                                                                           #
# Parameters:                                                               #
#	fichier_vocab : file name of the vocabulary                         #
#                                                                           #
# ######################################################################### #
 



# ######################################################################### #
#                                                                           #
# fonction LoadVocab                                                        #
# 	Charge en mmoire un vocabulaire (fichier contenant un mot par      #
#       ligne tri dans l'ordre alpha-numerique ("sort" format POSIX)       #
#                                                                           #
# Paramtres:                                                               #
#       fichier_vocab est le nom du fichier du vocabulaire                  #
#                                                                           #
# Retour:                                                                   #
#       tableau tabvoc qui contient le vocabulaire                          #
#       renvoie la taille du tableau tabvoc (nb de mots du vocab)           #
#                                                                           #
# ######################################################################### #
function LoadVocab (fichier_vocab)
{
	ind = 0
	while ( getline < fichier_vocab )
	{
		# chaque ligne est stocke dans "tabvoc"
		tabvoc[ind]=$0
		ind++
	}
	return ind
}
# ######################################################################### #




# ######################################################################### #
#                                                                           #
# fonction IsOOV                                                            #
#	Recherche dichotomique d'un mot dans un tableau de mot ordonn      #
#                                                                           #
# Paramtres:                                                               #
#	tabvoc est un tableau de chaines de caractres                      #
#       mot est une chaine de caractre                                     #
#       ind est un entier qui reprsente la taille du tableau               #
#                                                                           #
# Retour:                                                                   #
#       0 si le mot est prsent, 1 s'il est absent                          #
#                                                                           #
# ######################################################################### #
function IsOOV (tabvoc,mot,ind)
{
	gauche = 0
	droit  = ind+1
	
	while ( gauche != droit )
	{
		median = int((droit+gauche)/2) 
		if ( tabvoc[median] > mot )
			droit = median
		else
		{
			if ( tabvoc[median] < mot )
				gauche = median+1
			else
			{
				return 0	
			}
		}
	}
	return 1
}
# ######################################################################### #
 



# ######################################################################### #
#                                                                           #
# fonction Analyse                                                          #
#	Analyse et segmente une chaine de caractres hors vocabulaire       #
#                                                                           #
# Paramtres:                                                               #
#       token est une chaine de caractres  analyser                       #
#       Lst est un tableau de tokens rsultat de l'analyse                  #
#                                                                           #
# Retour:                                                                   #
#       taille du tableau de tokens                                         #
#                                                                           #
# ######################################################################### #
function Analyse (token, Lst)
{
	# Recherche d'un dlimiteur dans le token.
	# Ce sont les cas o l'on dcoupe systmatiquement
	if ( match (token, /[\\",?!;:\(\)\&\/\`\%\*\[\]\\]/) != 0)
	{
		# Segmente en fonction des dlimiteurs trouvs
	    	token = gensub (/([\\",?!;:\(\)\&\/\`\%\*\[\]\\])/, " \\1 ", "g", token)
		return split(token, Lst, " ")
	}
	else
	{
		# Recherche d'une apostrophe
		if (match (token, /[']/ ) != 0)
		{
			# On dcoupe juste aprs l'apostrophe
			sub(/'/, "' ", token)
			nb=split(token, Lst, " ")
			return nb
		}
		else
		{
			# Recherche d'un point
			if ( match (token, /[\.]/ ) != 0)
			{
				# Cas particulier : ... en fin de mot
				if ( length(token) > 3 &&  token ~ /\.\.\.$/) 
				{
					substr(token, 0, length(token)-3)
					Lst[1] = token
					Lst[2] = "..."
					return 2
				}
				else
				{
					# Cas d'une lettre suivie d'un point a. b. et 
					# Cas des abrviations.
					# On laisse tel quel.
					if  ( ( length(token) < 3 ) || ( index(token,".") < 3 ) )
					{
						Lst[1] = token
						return 1
					}	
					else
					{
						nb = split (token, LstTmp, ".") 
						j = 1
						for (i=1; i<nb; i++)
						{
							Lst[j] = LstTmp[i]
							j++
							Lst[j] = "."
							j++
						}
						Lst[j] = LstTmp[i]
						return j
					}
				}

			}
			# Recherche d'un tiret mot1-mot2
			else
			{
				if ( token ~ /[-]/ )
				{
					# On crit mot1
					lindex = index(token, "-")
					Lst[1] = substr(token, 1, lindex-1)

					# On cherche -mot2 dans le lexique
					letoken = substr(token, lindex, length(token)-lindex+1)
					if (IsOOV(tabvoc,letoken,ind) == 0)
					{
						# On crit -mot2
						Lst[2] = letoken
						return 2
					}
					else
					{
						# On clate -mot2
						token = substr(letoken, 2, length(token)-1);
						Lst[2] = "-"
						Lst[3] = token
						return 3
					}
				}
				else
				{
					# Situation non reconnue ! 
					# On crit le token tel quel 
					Lst[1] = token
					return 1
				}
			}
		}
	}
}
# ######################################################################### #




# ######################################################################### #
#                                                                           #
# Programme Principal                                                       #
#                                                                           #
# ######################################################################### #

BEGIN{
	ind = LoadVocab(fichier_vocab)	# Charge le vocab
}


{
	if ( match($0, /<[\/A-Zsp:]+>/))
	{
		# Les lignes qui sont des balises doivent rester telles-quelles
		print
	}
	else
	{

		# Eclate des guillemets, les parenthses, des # et les virgules
		gsub(/"/, " \" ")
		gsub(/\))/, " \) ")
		gsub(/\(/, " \( ")
		gsub(/#/, " # ")
		gsub(/,/, " , ")

		# Eclate les dbuts de phrases colls
		gsub(/<s>[^ ]/, "<s> ")

		# Eclate les nombres colls  d'autres caractres
		$0 = gensub (/([0-9])([^0-9])/, "\\1 \\2", "g", $0)
		$0 = gensub (/([^0-9])([0-9])/, "\\1 \\2", "g", $0)

		# Eclate les points colls au dbut de mots
		$0 = gensub (/ \.([a-z])/, "\. \\1", "g", $0)

		# Regroupe les nombres spars par des espaces
		$0 = gensub (/([0-9]) ([0-9])/, "\\1\\2", "g", $0)

		# Segmente en fonction des espaces
		NbTokens = split($0, LstTokens, " ")

		# Analyse chacun des tokens pour le re-segmenter ou non	
		for (token=1; token<=NbTokens; token++)
		{
			if ( IsOOV(tabvoc,LstTokens[token],ind) == 0  || 
			     LstTokens[token] ~ /<?s>/ || 
			     match(LstTokens[token], /^[\.\-'\",?!;:\(\)&\/\`%\*\[\]\\\_]$/) != 0)
			{
				print LstTokens[token]
			}
			else
			{
				nb1 = Analyse(LstTokens[token], Lst)
				for (tok=1; tok<=nb1; tok++)
				{
					nb2 = Analyse(Lst[tok], L)
					for (t=1; t<=nb2; t++) 
						print L[t]
				}
			}
		}
	}

}


