/******************************************************************************/
/*                                                                            */
/*                                  script.h                                  */
/*                                                                            */
/*                        Audio Segmentation Library                          */
/*                                                                            */
/* Guig                                                       Fri Nov 15 2002 */
/* -------------------------------------------------------------------------- */
/*  Copyright (C) 2002  IRISA                                                 */
/*                                                                            */
/*  This program is free software; you can redistribute it and/or             */
/*  modify it under the terms of the GNU General Public License               */
/*  as published by the Free Software Foundation; either version 2            */
/*  of the License, or (at your option) any later version.                    */
/*                                                                            */
/*  This program is distributed in the hope that it will be useful,           */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/*  GNU General Public License for more details.                              */
/*                                                                            */
/*  You should have received a copy of the GNU General Public License         */
/*  along with this program; if not, write to the Free Software               */
/*  Foundation, Inc., 59 Temple Place - Suite 330,                            */
/*  Boston, MA  02111-1307, USA.                                              */
/*                                                                            */
/******************************************************************************/
/*
 * CVS log:
 *
 *   $Author: ggravier $
 *   $Date: 2003/11/20 15:50:58 $
 *   $Revision: 1.3 $
 *
 */

/*
 * Script related functions.
 *
 * A script is a text file containing a list of data files, optionnally with
 * the corresponding segmentation file. The script file must contain one line
 * per data file according to the following syntax
 *
 *   /path/to/spro/data/file [path/to/the/corresponding/label/file] [# comment]
 *
 * where the label file is optionnal. Comments after each line can be added
 * using a # character. Please note that empty lines are not allowed.
 *
 * From the machine point of view, a script is a list of data filenames, a
 * list of corresponding segmentation filenames and the corresponding 
 * segmentations. The function load_script() reads a script from file
 * while make_script() makes a script from a list of data filenames with
 * NULL corresponding segmentation filenames.
 */

#ifndef _script_h_
# define _script_h_

# include <audioseg.h>

     /* ---------------------------- */
     /* ----- type definitions ----- */
     /* ---------------------------- */

typedef struct {
  unsigned long nfiles;            /* number of training files              */
  char **datfn;                    /* list of training filenames            */
  char **segfn;                    /* list of segmentation filenames        */
  asseg_t **seg;                   /* list of segmentations                 */
} script_t;

     /* ------------------------------- */
     /* ----- function prototypes ----- */
     /* ------------------------------- */

/* load script from file */
script_t *load_script(
  const char *,                    /* input filename                        */
  int                              /* load segmentations?                   */
);

/* make script from a list of data filenames */
script_t *make_script(
  int,		                   /* number of input filenames             */
  char **                          /* input filename                        */
);

/* free script */
void free_script(
  script_t *                       /* script address                        */
);

/* load segmentations */
asseg_t **load_script_seg(
  script_t *                       /* script address                        */
);

#endif /* _script_h_ */
