#!/bin/sh -x
#
# Test sgmlike program
#
# $Author: ggravier $
# $Date: 2003/08/22 14:15:12 $
# $Revision: 1.3 $
#


prog=sgmlike

echo "checking ${prog}:"

if test ! -x ../src/${prog}; then
  echo "  you should probably build ${prog} before running test..."
  exit 1
fi

mkdir -p log

#
# check usage 
#
../src/${prog} --help > log/${prog}.stdout 2> log/${prog}.stderr
diff log/${prog}.stdout ref/${prog}.usage > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  usage test passed"
else
  echo "  usage test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# global scoring test
#
../src/${prog} dat/know1.gmm dat/know1.lfcc > log/${prog}.stdout 2> log/${prog}.stderr
diff log/${prog}.stdout  ref/like.1 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  scoring test 1 passed"
else
  echo "  scoring test 1 failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# script and normalization test
#
../src/${prog} -n -L dat/script.lst dat/know1.gmm > log/${prog}.stdout 2> log/${prog}.stderr
diff log/${prog}.stdout  ref/like.2 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  scoring test 2 passed"
else
  echo "  scoring test 2 failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# segmentation scoring test
#
../src/${prog} -sdat/know.seg dat/know1.gmm dat/know1.lfcc > log/${prog}.stdout 2> log/${prog}.stderr
diff log/${prog}.stdout  ref/like.3 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  scoring test 3 passed"
else
  echo "  scoring test 3 failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# chopping test
#
../src/${prog} -sdat/know.seg -x spk1 dat/know1.gmm dat/know1.lfcc > log/${prog}.stdout 2> log/${prog}.stderr
diff log/${prog}.stdout  ref/like.4 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  scoring test 4 passed"
else
  echo "  scoring test 4 failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# model list and posterior test
#
../src/${prog} -p -f dat/models.lst dat/know1.lfcc > log/${prog}.stdout 2> log/${prog}.stderr
diff log/${prog}.stdout  ref/like.5 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  scoring test 5 passed"
else
  echo "  scoring test 5 failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# fixed length segment and N-best scoring test
#
../src/${prog} -b 2 -l 5.0 -f dat/models.lst dat/know1.lfcc > log/${prog}.stdout 2> log/${prog}.stderr
diff log/${prog}.stdout  ref/like.6 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  scoring test 6 passed"
else
  echo "  scoring test 6 failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

rmdir log